/*
 * Mellanox SX-1400/SX-1710/SN-2700 Platform Support.
 *
 * Author: Support (support@cumulusnetworks.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *  02110-1301, USA.
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/platform_data/at24.h>
#include <linux/i2c/pmbus.h>

#include "platform_defs.h"
#include "mlnx-mux-drv.h"

#define DRIVER_NAME	"mlx_sx1000_sn2000"
#define DRIVER_VERSION	"0.2"

#define MLX_SNX_LPCI2C_BUS_NAME     "lpci2c"

enum {
	LPC_I2C_BUS=0,
	LPC_I2C_MUX_BUS0=2,
	LPC_I2C_MUX_BUS1,
	LPC_I2C_MUX_BUS2,
	LPC_I2C_MUX_BUS3,
	LPC_I2C_MUX_BUS4,
	LPC_I2C_MUX_BUS5,
	LPC_I2C_MUX_BUS6,
	LPC_I2C_MUX_BUS7,
	LPC_I2C_MUX_BUS8,
	LPC_I2C_MUX_BUS9,
	LPC_I2C_MUX_BUS10,
	LPC_I2C_MUX_BUS11,
	LPC_I2C_MUX_BUS12,
	LPC_I2C_MUX_BUS13,
	LPC_I2C_MUX_BUS14,
	LPC_I2C_MUX_BUS15,
	LPC_I2C_MUX_BUS16,
};

static struct cpld_mux_platform_mode cpld_mux1_modes[] =
{
    {
        .adap_id = LPC_I2C_MUX_BUS0,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS1,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS2,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS3,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS4,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS5,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS6,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS7,
        .deselect_on_exit = 1,
    }
};
static struct cpld_mux_platform_mode cpld_mux2_modes[] =
{
    {
        .adap_id = LPC_I2C_MUX_BUS8,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS9,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS10,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS11,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS12,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS13,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS14,
        .deselect_on_exit = 1,
    },
    {
        .adap_id = LPC_I2C_MUX_BUS15,
        .deselect_on_exit = 1,
    }
};

static struct cpld_mux_platform_data cpld_mux1_data =
{
    .modes = cpld_mux1_modes,
    .num_modes = 8,
    .id = 0x25db,
    .sel_reg_addr = 0x25db,
    .first_channel = LPC_I2C_MUX_BUS0,
    .addr = 0x25db,
};

static struct cpld_mux_platform_data cpld_mux2_data =
{
    .modes = cpld_mux2_modes,
    .num_modes = 8,
    .id = 0x25da,
    .sel_reg_addr = 0x25da,
    .first_channel = LPC_I2C_MUX_BUS8,
    .addr = 0x25da,
};

/*
 * Mellanox board eeprom uses 16-bit address
 */
mk_eeprom(board, 51, 1024, AT24_FLAG_IRUGO | AT24_FLAG_ADDR16);
mk_eeprom(mainboard, 51, 4096, AT24_FLAG_IRUGO);
mk_eeprom(psu1, 51, 256, AT24_FLAG_IRUGO);
mk_eeprom(psu2, 50, 256, AT24_FLAG_IRUGO);

struct mlx_snx_device_info {
	int    bus;
	struct i2c_board_info board_info;
};

static struct pmbus_platform_data psu1_data = {
	.flags = PMBUS_SKIP_STATUS_CHECK,
};

static struct pmbus_platform_data psu2_data = {
	.flags = PMBUS_SKIP_STATUS_CHECK,
};

static struct mlx_snx_device_info mlx_snx_devices[] = {
	mk_i2cdev_wflags(LPC_I2C_BUS, "cpld_mux_tor", 0x25db & 0xff, &cpld_mux1_data, I2C_CLIENT_TEN),
	mk_i2cdev_wflags(LPC_I2C_BUS, "cpld_mux_tor", 0x25da & 0xff, &cpld_mux2_data, I2C_CLIENT_TEN),
	mk_i2cdev(LPC_I2C_BUS, "mlx_1000_2000_cpld", 0x60, NULL),
	mk_i2cdev(LPC_I2C_MUX_BUS3,  "mlnx-a2d-swb-drv", 0x6d, NULL),
	mk_i2cdev(LPC_I2C_MUX_BUS5,  "lm75", 0x4a, NULL),
	mk_i2cdev(LPC_I2C_MUX_BUS6,  "24c00", 0x51, &board_51_at24),
	mk_i2cdev(LPC_I2C_MUX_BUS8,  "24c01", 0x51, &psu1_51_at24),
	mk_i2cdev(LPC_I2C_MUX_BUS8,  "24c01", 0x50, &psu2_50_at24),
	mk_i2cdev(LPC_I2C_MUX_BUS8,  "pmbus", 0x59, &psu1_data),
	mk_i2cdev(LPC_I2C_MUX_BUS8,  "pmbus", 0x58, &psu2_data),
	mk_i2cdev(LPC_I2C_MUX_BUS13, "mlnx-a2d-mnb-drv", 0x6d, NULL),
	mk_i2cdev(LPC_I2C_MUX_BUS15, "lm75",  0x49, NULL),
};
static struct i2c_client *mlx_snx_clients_list[ARRAY_SIZE(mlx_snx_devices)];

static struct i2c_adapter *get_adapter(int bus)
{
	int bail=10;
	struct i2c_adapter *adapter;

	for (; bail; bail--) {
		adapter = i2c_get_adapter(bus);
		if (adapter) {
			return adapter;
		}
		msleep(100);
	}
	return NULL;
}

static int get_bus_by_name(char *name)
{
     struct i2c_adapter *adapter;
     int i;

     for (i = 0; i < LPC_I2C_MUX_BUS0; i++) {
	  adapter = get_adapter(i);
	  if (adapter) {
	       if (strncmp(adapter->name, name, strlen(name)) == 0) {
		    i2c_put_adapter(adapter);
		    return i;
	       }
	       i2c_put_adapter(adapter);
	  }
     }
     pr_err("could not find %s bus.\n", name);
     return -1;
}

static struct i2c_client *add_i2c_client(int bus,
                                         struct i2c_board_info *board_info)
{
	struct i2c_adapter *adapter;
	struct i2c_client *client;

	adapter = get_adapter(bus);
	if (!adapter) {
		pr_err("could not get adapter %u\n", bus);
		return ERR_PTR(-ENODEV);
	}
	client = i2c_new_device(adapter, board_info);
	if (!client) {
		pr_err("could not add device\n");
		return ERR_PTR(-ENODEV);
	}
	return client;
}

static int __init mlx_sx1000_sn2000_i2c_init(void)
{
	int i;
	int ret;
	int bus_num;
	struct i2c_client *client;

	ret = 0;
	for (i = 0; i < ARRAY_SIZE(mlx_snx_devices); i++) {
		bus_num = mlx_snx_devices[i].bus;
		if (bus_num == LPC_I2C_BUS)
			bus_num = get_bus_by_name(MLX_SNX_LPCI2C_BUS_NAME);
		if (bus_num < 0) {
			ret = -ENODEV;
			goto exit;
		}
		client = add_i2c_client(bus_num,
					&mlx_snx_devices[i].board_info);
		if (IS_ERR(client)) {
			ret = PTR_ERR(client);
			break;
		}
		mlx_snx_clients_list[i] = client;
	}

exit:
	return ret;
}

static void free_mlx_snx_clients(void)
{
	int i, idx;
	for (i = ARRAY_SIZE(mlx_snx_devices); i; i--) {
		idx = i - 1;
		if (mlx_snx_clients_list[idx])
			i2c_unregister_device(mlx_snx_clients_list[idx]);
	}
}

static int __exit mlx_sx1000_sn2000_i2c_exit(void)
{
	free_mlx_snx_clients();
	return 0;
}

static int mlx_sx1000_sn2000_probe(struct platform_device *dev)
{
	return 0;
}

static int mlx_sx1000_sn2000_remove(struct platform_device *dev)
{
	return 0;
}

static struct platform_driver mlx_sx1000_sn2000_driver = {
	.driver = {
		.name = "mlx_sx1000_sn2000",
		.owner = THIS_MODULE,
	},
	.probe = mlx_sx1000_sn2000_probe,
	.remove = mlx_sx1000_sn2000_remove,
};

static struct platform_device *mlx_sx1000_sn2000_device;

static int __init mlx_sx1000_sn2000_init(void)
{
	int ret;

	ret = mlx_sx1000_sn2000_i2c_init();
	if (ret) {
		pr_err("Initializing I2C subsystem failed\n");
		goto err_drvr;
	}

	ret = platform_driver_register(&mlx_sx1000_sn2000_driver);
	if (ret) {
		pr_err("platform_driver_register() failed for cpld device");
		goto err_drvr;
	}

	mlx_sx1000_sn2000_device = platform_device_alloc("mlx_sx1000_sn2000", 0);
	if (!mlx_sx1000_sn2000_device) {
		pr_err("platform_device_alloc() failed for mlx_sx1000_sn2000 device");
		ret = -ENOMEM;
		goto err_dev_alloc;
	}

	ret = platform_device_add(mlx_sx1000_sn2000_device);
	if (ret) {
		pr_err("platform_device_add() failed for mlx_sx1000_sn2000 device.\n");
		goto err_dev_add;
	}

	pr_info(DRIVER_NAME": version "DRIVER_VERSION" successfully loaded\n");
	return 0;

err_dev_add:
	platform_device_put(mlx_sx1000_sn2000_device);

err_dev_alloc:
	platform_driver_unregister(&mlx_sx1000_sn2000_driver);

err_drvr:
	free_mlx_snx_clients();
	return ret;
}

static void __exit mlx_sx1000_sn2000_exit(void)
{
	platform_driver_unregister(&mlx_sx1000_sn2000_driver);
	platform_device_unregister(mlx_sx1000_sn2000_device);
	mlx_sx1000_sn2000_i2c_exit();

	pr_info(DRIVER_NAME": version "DRIVER_VERSION" unloaded\n");
}

module_init(mlx_sx1000_sn2000_init);
module_exit(mlx_sx1000_sn2000_exit);

MODULE_AUTHOR("Support (support@cumulusnetworks.com)");
MODULE_DESCRIPTION("Mellanox SX-1400/SX-1710/SN-2700 Platform Support");
MODULE_VERSION(DRIVER_VERSION);
MODULE_LICENSE("GPL");
